@@ -1047,6 +1047,7 @@ on_build_terminated (AnjutaLauncher *launcher,
 							   _("Command terminated for an unknown reason"));
 		}
 		build_program_callback (context->program, G_OBJECT (context->plugin), context, err);
+		if (err != NULL) g_error_free (err);
 	}
 	if (context->used)
 		return;	/* Another command is run */
@@ -1241,6 +1242,7 @@ build_get_context (BasicAutotoolsPlugin *plugin, const gchar *dir,
 {
 	BuildContext *context = NULL;
 	AnjutaPluginManager *plugin_manager;
+	gchar *buf;
 
 	if (with_view)
 	{
@@ -1277,8 +1279,9 @@ build_get_context (BasicAutotoolsPlugin *plugin, const gchar *dir,
 	g_signal_connect (G_OBJECT (context->launcher), "child-exited",
 					  G_CALLBACK (on_build_terminated), context);
 	build_context_push_dir (context, "default", dir);
-	dir = g_strconcat (dir, "/", NULL);
-	g_chdir (dir);
+	buf = g_strconcat (dir, "/", NULL);
+	g_chdir (buf);
+	g_free (buf);
 	
 	plugin->contexts_pool = g_list_append (plugin->contexts_pool, context);
 	
@@ -1288,8 +1291,6 @@ build_get_context (BasicAutotoolsPlugin *plugin, const gchar *dir,
 static void
 build_set_command_in_context (BuildContext* context, BuildProgram *prog)
 {
-	if (context->program != NULL)
-		build_program_free (context->program);
 	context->program = prog;
 	context->used = TRUE;
 }
@@ -1320,9 +1321,6 @@ build_execute_command_in_context (BuildContext* context, GError **err)
 	
 	build_program_override (context->program, context->environment);
 	
-	/* Add current directory */
-	build_program_add_env (context->program, "PWD", context->program->work_dir);
-	
 	if (context->message_view)
 	{
 		gchar *command;
@@ -1337,21 +1335,75 @@ build_execute_command_in_context (BuildContext* context, GError **err)
 		g_free (command);
 	
 		anjuta_launcher_execute_v (context->launcher,
-								   context->program->argv,
-								   context->program->envp,
-								   on_build_mesg_arrived, context);
+		    context->program->work_dir,
+		    context->program->argv,
+		    context->program->envp,
+		    on_build_mesg_arrived, context);
 	}
 	else
 	{
 		anjuta_launcher_execute_v (context->launcher,
-								   context->program->argv,
-								   context->program->envp,
-								   NULL, NULL);
+		    context->program->work_dir,
+		    context->program->argv,
+		    context->program->envp,
+		    NULL, NULL);
 	}		
 	
 	return TRUE;
 }	
 
+static void
+build_delayed_execute_command (IAnjutaFileSavable *savable, GFile *file, gpointer user_data)
+{
+	BuildContext *context = (BuildContext *)user_data;
+
+	if (savable != NULL)
+	{
+		g_signal_handlers_disconnect_by_func (savable, G_CALLBACK (build_delayed_execute_command), user_data); 
+		context->file_saved--;
+	}	
+	
+	if (context->file_saved == 0)
+	{
+		build_execute_command_in_context (context, NULL);
+	}
+}
+
+static gboolean
+build_save_and_execute_command_in_context (BuildContext* context, GError **err)
+{
+	IAnjutaDocumentManager *docman;
+
+	context->file_saved = 0;
+	
+	docman = anjuta_shell_get_interface (context->plugin->shell, IAnjutaDocumentManager, NULL);
+	/* No document manager, so no file to save */
+	if (docman != NULL)
+	{
+		GList *doc_list = ianjuta_document_manager_get_doc_widgets (docman, NULL);
+		GList *node;
+
+		for (node = g_list_first (doc_list); node != NULL; node = g_list_next (node))
+		{
+			if (IANJUTA_IS_FILE_SAVABLE (node->data))
+			{
+				IAnjutaFileSavable* save = IANJUTA_FILE_SAVABLE (node->data);
+				if (ianjuta_file_savable_is_dirty (save, NULL))
+				{
+					context->file_saved++;
+					g_signal_connect (G_OBJECT (save), "saved", G_CALLBACK (build_delayed_execute_command), context);
+					ianjuta_file_savable_save (save, NULL);
+				}
+			}
+		}
+		g_list_free (doc_list);		
+	}
+	
+	build_delayed_execute_command (NULL, NULL, context);
+
+	return TRUE;
+}
+
 static gchar *
 build_dir_from_source (BasicAutotoolsPlugin *plugin, const gchar *src_dir)
 {
@@ -1392,60 +1444,81 @@ build_execute_command (BasicAutotoolsPlugin* bplugin, BuildProgram *prog,
 	}
 }
 
-static void
-build_delayed_execute_command (IAnjutaFileSavable *savable, GFile *file, gpointer user_data)
-{
-	BuildContext *context = (BuildContext *)user_data;
-
-	if (savable != NULL)
-	{
-		g_signal_handlers_disconnect_by_func (savable, G_CALLBACK (build_delayed_execute_command), user_data); 
-		context->file_saved--;
-	}	
-	
-	if (context->file_saved == 0)
-	{
-		build_execute_command_in_context (context, NULL);
-	}
-}
-
 static BuildContext*
 build_save_and_execute_command (BasicAutotoolsPlugin* bplugin, BuildProgram *prog,
 								gboolean with_view, GError **err)
 {
-	AnjutaPlugin* plugin = ANJUTA_PLUGIN(bplugin);
 	BuildContext *context;
-	IAnjutaDocumentManager *docman;
+	
+	context = build_get_context (bplugin, prog->work_dir, with_view);
+
+	build_set_command_in_context (context, prog);
+	if (!build_save_and_execute_command_in_context (context, err))
+	{
+		build_context_destroy (context);
+		context = NULL;
+	}
+
+	return context;
+}
+
+static void
+build_execute_after_command (GObject *sender,
+							   IAnjutaBuilderHandle handle,
+							   GError *error,
+							   gpointer user_data)
+{
+	BuildProgram *prog = (BuildProgram *)user_data;
+	BuildContext *context = (BuildContext *)handle;
+
+	/* Try next command even if the first one fail (make distclean return an error) */
+	if ((error == NULL) || (error->code != IANJUTA_BUILDER_ABORTED))
+	{
+		build_set_command_in_context (context, prog);
+		build_execute_command_in_context (context, NULL);
+	}
+	else
+	{
+		build_program_free (prog);
+	}
+}
+
+static BuildContext*
+build_save_distclean_and_execute_command (BasicAutotoolsPlugin* bplugin, BuildProgram *prog,
+								gboolean with_view, GError **err)
+{
+	BuildContext *context;
 
 	context = build_get_context (bplugin, prog->work_dir, with_view);
-	build_set_command_in_context (context, prog);
-	context->file_saved = 0;
-
-	docman = anjuta_shell_get_interface (plugin->shell, IAnjutaDocumentManager, NULL);
-	/* No document manager, so no file to save */
-	if (docman != NULL)
+	
+	if ((strcmp (prog->work_dir, bplugin->project_root_dir) != 0) && directory_has_file (bplugin->project_root_dir, "config.status"))
 	{
-		GList *doc_list = ianjuta_document_manager_get_doc_widgets (docman, NULL);
-		GList *node;
-
-		for (node = g_list_first (doc_list); node != NULL; node = g_list_next (node))
+		BuildProgram *new_prog;
+		
+		// Need to run make clean before
+		if (!anjuta_util_dialog_boolean_question (GTK_WINDOW (ANJUTA_PLUGIN (bplugin)->shell), _("Before using this new configuration, the default one needs to be removed. Do you want to do that ?"), NULL))
 		{
-			if (IANJUTA_IS_FILE_SAVABLE (node->data))
-			{
-				IAnjutaFileSavable* save = IANJUTA_FILE_SAVABLE (node->data);
-				if (ianjuta_file_savable_is_dirty (save, NULL))
-				{
-					context->file_saved++;
-					g_signal_connect (G_OBJECT (save), "saved", G_CALLBACK (build_delayed_execute_command), context);
-					ianjuta_file_savable_save (save, NULL);
-				}
-			}
+			GError *err;
+			
+			err = g_error_new (ianjuta_builder_error_quark (),
+									 IANJUTA_BUILDER_CANCELED,
+									 _("Command canceled by user"));
+			
+			build_program_callback (context->program, G_OBJECT (bplugin), context, err);
+			
+			return NULL;
 		}
-		g_list_free (doc_list);		
+		new_prog = build_program_new_with_command (bplugin->project_root_dir,
+										   "%s",
+										   CHOOSE_COMMAND (bplugin, DISTCLEAN));
+		build_program_set_callback (new_prog, build_execute_after_command, prog);
+		prog = new_prog;
 	}
 	
-	build_delayed_execute_command (NULL, NULL, context);
-	
+	build_set_command_in_context (context, prog);
+
+	build_save_and_execute_command_in_context (context, NULL);
+
 	return context;
 }
 
@@ -1671,6 +1744,8 @@ build_compile_file (BasicAutotoolsPlugin *plugin, const gchar *filename)
 		g_hash_table_insert (target_ext, ".cxx", ".o");
 		g_hash_table_insert (target_ext, ".c++", ".o");
 		g_hash_table_insert (target_ext, ".cc", ".o");
+		g_hash_table_insert (target_ext, ".y", ".o");
+		g_hash_table_insert (target_ext, ".l", ".o");
 		g_hash_table_insert (target_ext, ".in", "");
 		g_hash_table_insert (target_ext, ".in.in", ".in");
 		g_hash_table_insert (target_ext, ".la", ".la");
@@ -1777,7 +1852,7 @@ build_configure_dir (BasicAutotoolsPlugin *plugin, const gchar *dirname, const g
 {
 	BuildContext *context;
 	BuildProgram *prog;
-	BuildConfigureAndBuild *pack = g_new(BuildConfigureAndBuild, 1);
+	BuildConfigureAndBuild *pack = g_new0 (BuildConfigureAndBuild, 1);
 
 	
 	prog = build_program_new_with_command (dirname,
@@ -1791,7 +1866,7 @@ build_configure_dir (BasicAutotoolsPlugin *plugin, const gchar *dirname, const g
 	pack->name = g_strdup (name);
 	build_program_set_callback (prog, build_project_configured, pack);
 	
-	context = build_save_and_execute_command (plugin, prog, TRUE, NULL);
+	context = build_save_distclean_and_execute_command (plugin, prog, TRUE, NULL);
 	
 	return context;
 }
@@ -1822,7 +1897,7 @@ build_configure_after_autogen (GObject *sender,
 		{
 			gboolean older;
 			
-			filename = g_build_filename (context->program->work_dir, "config.log", NULL);
+			filename = g_build_filename (context->program->work_dir, "config.status", NULL);
 			older =(stat (filename, &log_stat) != 0) || (log_stat.st_mtime < conf_stat.st_mtime);
 			g_free (filename);
 			
@@ -1867,7 +1942,7 @@ build_generate_dir (BasicAutotoolsPlugin *plugin, const gchar *dirname, const gc
 {
 	BuildContext *context;
 	BuildProgram *prog;
-	BuildConfigureAndBuild *pack = g_new (BuildConfigureAndBuild, 1);
+	BuildConfigureAndBuild *pack = g_new0 (BuildConfigureAndBuild, 1);
 	
 	if (directory_has_file (plugin->project_root_dir, "autogen.sh"))
 	{
@@ -1890,7 +1965,7 @@ build_generate_dir (BasicAutotoolsPlugin *plugin, const gchar *dirname, const gc
 	pack->name = g_strdup (name);
 	build_program_set_callback (prog, build_configure_after_autogen, pack);
 	
-	context = build_save_and_execute_command (plugin, prog, TRUE, NULL);
+	context = build_save_distclean_and_execute_command (plugin, prog, TRUE, NULL);
 	
 	return context;
 }
@@ -1902,16 +1977,19 @@ build_configure_dialog (BasicAutotoolsPlugin *plugin, BuildFunc func, const gcha
 	gboolean run_autogen = FALSE;
 	const gchar *project_root;
 	GValue value = {0,};
+	const gchar *old_config_name;
 	
 	run_autogen = !directory_has_file (plugin->project_root_dir, "configure");
 	
 	anjuta_shell_get_value (ANJUTA_PLUGIN (plugin)->shell, IANJUTA_PROJECT_MANAGER_PROJECT_ROOT_URI, &value, NULL);
 	project_root = g_value_get_string (&value);
 	parent = GTK_WINDOW (ANJUTA_PLUGIN(plugin)->shell);
-	
+
+	old_config_name = build_configuration_get_name (build_configuration_list_get_selected (plugin->configurations));
 	if (build_dialog_configure (parent, project_root, plugin->configurations, &run_autogen))
 	{
 		BuildConfiguration *config;
+		BuildContext* context;
 		GFile *file;
 		gchar *uri;
 		gchar *build_dir;
@@ -1929,13 +2007,19 @@ build_configure_dialog (BasicAutotoolsPlugin *plugin, BuildFunc func, const gcha
 		
 		if (run_autogen)
 		{
-			build_generate_dir (plugin, build_dir, args, func, name);
+			context = build_generate_dir (plugin, build_dir, args, func, name);
 		}
 		else
 		{
-			build_configure_dir (plugin, build_dir, args, func, name);
+			context = build_configure_dir (plugin, build_dir, args, func, name);
 		}
 		g_free (build_dir);
+
+		if (context == NULL)
+		{
+			/* Restore previous configuration */
+			build_configuration_list_select (plugin->configurations, old_config_name);
+		}
 	}
 	
 }
@@ -1983,6 +2067,44 @@ build_configure_and_build (BasicAutotoolsPlugin *plugin, BuildFunc func, const g
 	}
 }
 
+/* Configuration commands
+ *---------------------------------------------------------------------------*/
+
+static GList*
+build_list_configuration (BasicAutotoolsPlugin *plugin)
+{
+	BuildConfiguration *cfg;
+	GList *list = NULL;
+	
+	for (cfg = build_configuration_list_get_first (plugin->configurations); cfg != NULL; cfg = build_configuration_next (cfg))
+	{
+		const gchar *name = build_configuration_get_name (cfg);
+
+		if (name != NULL) list = g_list_prepend (list, (gpointer)name);
+	}
+
+	return list;
+}
+
+static const gchar*
+build_get_uri_configuration (BasicAutotoolsPlugin *plugin, const gchar *uri)
+{
+	BuildConfiguration *cfg;
+	
+	for (cfg = build_configuration_list_get_first (plugin->configurations); cfg != NULL; cfg = build_configuration_next (cfg))
+	{
+		const gchar *root = build_configuration_list_get_build_uri  (plugin->configurations, cfg);
+
+		if (strncmp (uri, root, strlen (root)) == 0)
+		{
+			return build_configuration_get_name (cfg);
+		}
+	}
+
+	return NULL;
+}
+
+
 /* User actions
  *---------------------------------------------------------------------------*/
 
@@ -2930,7 +3052,8 @@ value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 		filename = g_file_get_path(file);
 		g_object_unref (file);
 		g_return_if_fail (filename != NULL);
-		ba_plugin->current_editor_filename = filename;
+		ba_plugin->current_editor_filename = anjuta_util_get_real_path (filename);
+		g_free (filename);
 		update_module_ui (ba_plugin);
 	}
 	g_idle_add (on_update_indicators_idle, plugin);
@@ -3332,67 +3455,70 @@ ibuilder_cancel (IAnjutaBuilder *builder, IAnjutaBuilderHandle handle, GError **
 	build_cancel_command (plugin, (BuildContext *)handle, err);
 }
 
+static GList*
+ibuilder_list_configuration (IAnjutaBuilder *builder, GError **err)
+{
+	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (builder);
+
+	return build_list_configuration (plugin);
+}
+
+static const gchar*
+ibuilder_get_uri_configuration (IAnjutaBuilder *builder, const gchar *uri, GError **err)
+{
+	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (builder);
+
+	return build_get_uri_configuration (plugin, uri);
+}
+
 static void
 ibuilder_iface_init (IAnjutaBuilderIface *iface)
 {
 	iface->is_built = ibuilder_is_built;
 	iface->build = ibuilder_build;
 	iface->cancel = ibuilder_cancel;
+	iface->list_configuration = ibuilder_list_configuration;
+	iface->get_uri_configuration = ibuilder_get_uri_configuration;
 }
 
 /* IAnjutaPreferences implementation
  *---------------------------------------------------------------------------*/
 
-static GtkBuilder *bxml;
-
 static void
 ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
 {
-	GError* error = NULL;
 	GtkWidget *root_check;
-	GtkWidget *entry;
+	GtkWidget *make_check;
+	GtkWidget *root_entry;
+	GtkWidget *make_entry;
+	GtkBuilder *bxml;
 		
 	/* Create the preferences page */
-	bxml =  gtk_builder_new();
+	bxml =  anjuta_util_builder_new (BUILDER_FILE, NULL);
+	if (!bxml) return;
 
-	if (!gtk_builder_add_from_file (bxml, BUILDER_FILE, &error))
-	{
-		g_warning ("Couldn't load builder file: %s", error->message);
-		g_error_free (error);
-	}
+	anjuta_util_builder_get_objects (bxml,
+	    INSTALL_ROOT_CHECK, &root_check,
+	    INSTALL_ROOT_ENTRY, &root_entry,
+	    PARALLEL_MAKE_CHECK, &make_check,
+	    PARALLEL_MAKE_SPIN, &make_entry,
+	    NULL);
+	
+	g_signal_connect(G_OBJECT(root_check), "toggled", G_CALLBACK(on_root_check_toggled), root_entry);
+	on_root_check_toggled (root_check, root_entry);
 
-	root_check = GTK_WIDGET (gtk_builder_get_object (bxml, INSTALL_ROOT_CHECK));
-	entry = GTK_WIDGET (gtk_builder_get_object (bxml, INSTALL_ROOT_ENTRY));
-	g_signal_connect(G_OBJECT(root_check), "toggled", G_CALLBACK(on_root_check_toggled), entry);
-	on_root_check_toggled (root_check, entry);
-
-	root_check = GTK_WIDGET (gtk_builder_get_object (bxml, PARALLEL_MAKE_CHECK));
-	entry = GTK_WIDGET (gtk_builder_get_object (bxml, PARALLEL_MAKE_SPIN));
-	g_signal_connect(G_OBJECT(root_check), "toggled", G_CALLBACK(on_root_check_toggled), entry);
-	on_root_check_toggled (root_check, entry);
+	g_signal_connect(G_OBJECT(make_check), "toggled", G_CALLBACK(on_root_check_toggled), make_entry);
+	on_root_check_toggled (make_check, make_entry);
 	
 	anjuta_preferences_add_from_builder (prefs, bxml, BUILD_PREFS_ROOT, _("Build Autotools"),  ICON_FILE);
+	
+	g_object_unref (bxml);
 }
 
 static void
 ipreferences_unmerge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
 {
-	GtkWidget *root_check;
-	GtkWidget *entry;
-
-	root_check = GTK_WIDGET (gtk_builder_get_object (bxml, INSTALL_ROOT_CHECK));
-	entry = GTK_WIDGET (gtk_builder_get_object (bxml, INSTALL_ROOT_ENTRY));
-	g_signal_handlers_disconnect_by_func(G_OBJECT(root_check),
-		G_CALLBACK(on_root_check_toggled), entry);
-
-	root_check = GTK_WIDGET (gtk_builder_get_object (bxml, PARALLEL_MAKE_CHECK));
-	entry = GTK_WIDGET (gtk_builder_get_object (bxml, PARALLEL_MAKE_SPIN));
-	g_signal_handlers_disconnect_by_func(G_OBJECT(root_check),
-		G_CALLBACK(on_root_check_toggled), entry);
-		
 	anjuta_preferences_remove_page(prefs, _("Build Autotools"));
-	
-	g_object_unref (bxml);
 }
 
 static void
