@@ -20,6 +20,7 @@
 
 #include <config.h>
 #include <ctype.h>
+#include <stdlib.h>
 #include <libanjuta/anjuta-shell.h>
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-iterable.h>
@@ -43,22 +44,22 @@
 #define ANJUTA_PIXMAP_COMPLETE			  "anjuta-complete"
 #define ANJUTA_PIXMAP_AUTOCOMPLETE        "anjuta-complete-auto"
 #define ANJUTA_PIXMAP_AUTOINDENT          "anjuta-indent-auto"
-#define ANJUTA_STOCK_SWAP                     "anjuta-swap"
+#define ANJUTA_STOCK_SWAP                 "anjuta-swap"
 #define ANJUTA_STOCK_COMPLETE         	  "anjuta-complete"
 #define ANJUTA_STOCK_AUTOCOMPLETE         "anjuta-autocomplete"
 #define ANJUTA_STOCK_AUTOINDENT           "anjuta-indent"
 
-#define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-language-support-cpp-java.ui"
-#define PREFS_GLADE PACKAGE_DATA_DIR"/glade/anjuta-language-cpp-java.glade"
+#define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-language-support-cpp-java.xml"
+#define PREFS_BUILDER PACKAGE_DATA_DIR"/glade/anjuta-language-cpp-java.ui"
 #define ICON_FILE "anjuta-language-cpp-java-plugin.png"
 
 /* Preferences keys */
 
 #define PREF_INDENT_AUTOMATIC "language.cpp.indent.automatic"
-#define PREF_INDENT_ADAPTIVE "language.cpp.indent.adaptive"
-#define PREF_INDENT_TAB_INDENTS "language.cpp.indent.tab.indents"
 #define PREF_INDENT_STATEMENT_SIZE "language.cpp.indent.statement.size"
 #define PREF_INDENT_BRACE_SIZE "language.cpp.indent.brace.size"
+#define PREF_INDENT_PARANTHESE_LINEUP "language.cpp.indent.paranthese.lineup"
+#define PREF_INDENT_PARANTHESE_SIZE "language.cpp.indent.paranthese.size"
 #define PREF_BRACE_AUTOCOMPLETION "language.cpp.brace.autocompletion"
 
 #define TAB_SIZE (ianjuta_editor_get_tabsize (editor, NULL))
@@ -178,42 +179,39 @@ get_line_indentation (IAnjutaEditor *editor, gint line_num)
 		else
 			line_indent++;
 		idx++; /* Since we are looking for first non-space char, simple
-	            * increment of the utf8 chars would do */
+				* increment of the utf8 chars would do */
 	}
 	g_free (line_string);
 	return line_indent;
 }
 
 static gchar *
-get_line_indentation_string (IAnjutaEditor *editor, gint spaces)
+get_line_indentation_string (IAnjutaEditor *editor, gint spaces, gint line_indent_spaces)
 {
 	gint i;
-	
-	/* DEBUG_PRINT ("In %s()", __FUNCTION__); */
-	g_return_val_if_fail (spaces >= 0, NULL);
-	
-	if (spaces <= 0)
+	gchar *indent_string;
+		
+	if ((spaces + line_indent_spaces) <= 0)
 		return NULL;
 	
 	if (USE_SPACES_FOR_INDENTATION)
 	{
-		gchar *indent_string = g_new0 (gchar, spaces + 1);
-		for (i = 0; i < spaces; i++)
+		indent_string = g_new0 (gchar, spaces + line_indent_spaces + 1);
+		for (i = 0; i < (spaces + line_indent_spaces); i++)
 			indent_string[i] = ' ';
-		return indent_string;
 	}
 	else
 	{
 		gint num_tabs = spaces / TAB_SIZE;
 		gint num_spaces = spaces % TAB_SIZE;
-		gchar *indent_string = g_new0 (gchar, num_tabs + num_spaces + 1);
+		indent_string = g_new0 (gchar, num_tabs + num_spaces + line_indent_spaces + 1);
 		
 		for (i = 0; i < num_tabs; i++)
 			indent_string[i] = '\t';
-		for (; i < num_tabs + num_spaces; i++)
+		for (; i < num_tabs + (num_spaces + line_indent_spaces); i++)
 			indent_string[i] = ' ';
-		return indent_string;
 	}
+	return indent_string;
 }
 
 /* Sets the iter to line end of previous line and TRUE is returned.
@@ -587,6 +585,11 @@ initialize_indentation_params (CppJavaPlugin *plugin)
 	gboolean line_comment = FALSE;
 	gchar mini_buffer[MINI_BUFFER_SIZE] = {0};
 	
+	plugin->smart_indentation = anjuta_preferences_get_bool (plugin->prefs, PREF_INDENT_AUTOMATIC);
+	/* Disable editor intern auto-indent if smart indentation is enabled */
+	ianjuta_editor_set_auto_indent (IANJUTA_EDITOR(plugin->current_editor),
+								    !plugin->smart_indentation, NULL);
+
 	/* Initialize indentation parameters */
 	plugin->param_tab_size = -1;
 	plugin->param_statement_indentation = -1;
@@ -675,7 +678,7 @@ initialize_indentation_params (CppJavaPlugin *plugin)
 }
 
 static gint
-set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation)
+set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation, gint line_indent_spaces)
 {
 	IAnjutaIterable *line_begin, *line_end, *indent_position;
 	IAnjutaIterable *current_pos;
@@ -721,10 +724,10 @@ set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation)
 	//DEBUG_PRINT ("carat offset is = %d", carat_offset);
 	
 	/* Set new indentation */
-	if (indentation > 0)
+	if ((indentation + line_indent_spaces) > 0)
 	{
-		indent_string = get_line_indentation_string (editor, indentation);
-		nchars = g_utf8_strlen (indent_string, -1);
+		indent_string = get_line_indentation_string (editor, indentation, line_indent_spaces);
+		nchars = indent_string ? g_utf8_strlen (indent_string, -1) : 0;
 		
 		/* Only indent if there is something to indent with */
 		if (indent_string)
@@ -761,7 +764,7 @@ set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation)
 	/* If indentation == 0, we really didn't enter the previous code block,
 	 * but we may need to clear existing indentation.
 	 */
-	if (indentation == 0)
+	if ((indentation + line_indent_spaces) == 0)
 	{
 		/* Get existing indentation */
 		if (ianjuta_iterable_compare (indent_position, line_begin, NULL) > 0)
@@ -822,6 +825,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 						   IAnjutaEditor *editor,
 						   gint line_num,
 						   gint *incomplete_statement,
+						   gint *line_indent_spaces,
 						   gboolean *colon_indent)
 {
 	IAnjutaIterable *iter;
@@ -834,6 +838,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 	gboolean line_checked_for_comment = FALSE;
 	
 	*incomplete_statement = -1;
+	*line_indent_spaces = 0;
 	
 	if (line_num <= 1)
 		return 0;
@@ -843,7 +848,8 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 	iter = ianjuta_editor_get_line_begin_position (editor, line_num, NULL);
 	
 	current_line_is_preprocessor = line_is_preprocessor (editor, iter);
-	current_line_is_continuation = line_is_continuation (editor, iter);
+	current_line_is_continuation = 
+		line_is_continuation (editor, iter);
 	/*
 	DEBUG_PRINT ("Current line is preprocessor = %d",
 				 current_line_is_preprocessor);
@@ -856,7 +862,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 	{
 		/* Continuation of preprocessor line -- just maintain indentation */
 		g_object_unref (iter);
-		return get_line_indentation (editor, line_num - 1);;
+		return get_line_indentation (editor, line_num - 1);
 	}
 	else if (current_line_is_preprocessor)
 	{
@@ -1065,22 +1071,36 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 		else if (point_ch == '(' || point_ch == '[')
 		{
 			line_indent = 0;
-			while (ianjuta_iterable_previous (iter, NULL))
+			if (anjuta_preferences_get_bool (plugin->prefs,
+			                                 PREF_INDENT_PARANTHESE_LINEUP))
 			{
-				gchar dummy_ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (iter), 0,
-														 NULL);
-				if (iter_is_newline (iter, dummy_ch))
+				while (ianjuta_iterable_previous (iter, NULL))
 				{
-					skip_iter_to_newline_head (iter, dummy_ch);
-					break;
+					gchar dummy_ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (iter), 0,
+					                                               NULL);
+					if (iter_is_newline (iter, dummy_ch))
+					{
+						skip_iter_to_newline_head (iter, dummy_ch);
+						break;
+					}
+					if (dummy_ch == '\t')
+						line_indent += TAB_SIZE;
+					else
+						(*line_indent_spaces)++;
 				}
-				if (dummy_ch == '\t')
-					line_indent += TAB_SIZE;
-				else
-					line_indent++;
+				(*line_indent_spaces)++;
+				line_indent += extra_indent;
+			}
+			else
+			{
+				gint line_for_indent =
+					ianjuta_editor_get_line_from_position (editor, iter, NULL);
+				line_indent = get_line_indentation (editor, line_for_indent);
+				line_indent += extra_indent;
+
+				(*line_indent_spaces) += anjuta_preferences_get_int (plugin->prefs,
+				                                                     PREF_INDENT_PARANTHESE_SIZE);
 			}
-			line_indent += 1;
-			line_indent += extra_indent;
 			
 			/* Although statement is incomplete at this point, we don't
 			 * set it to incomplete and just leave it to unknown to avaoid
@@ -1175,7 +1195,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 		line_indent += extra_indent;
 	}
 	g_object_unref (iter);
-	
+		
 	return line_indent;
 }
 
@@ -1218,9 +1238,32 @@ is_iter_inside_string (IAnjutaIterable *iter)
 	return FALSE;
 }
 
+static gboolean
+spaces_only (IAnjutaEditor* editor, IAnjutaIterable* begin, IAnjutaIterable* end)
+{
+	gboolean empty = TRUE;
+	gchar* idx;
+	gchar* text = ianjuta_editor_get_text (editor, begin, end, NULL);
+
+	if (text == NULL)
+		return TRUE;
+	
+
+	for (idx = text; *idx != '\0'; idx++)
+	{
+		if (!isspace(*idx))
+		{
+			empty = FALSE;
+			break;
+		}
+	}
+	g_free(text);
+	return empty;
+}
+
 static gint
 get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
-						   gint line)
+						   gint line, gint *line_indent_spaces)
 {
 	IAnjutaIterable *iter;
 	IAnjutaIterable *end_iter;
@@ -1229,12 +1272,31 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 	gboolean colon_indent = FALSE;
 	
 	g_return_val_if_fail (line > 0, 0);
+
+	/* be sure to set a default if we're in the first line otherwise
+	 * the pointer'll be left hanging with no value.
+	 */
+	*line_indent_spaces = 0;
 	
 	if (line == 1) /* First line */
+	{
 		return 0;
+	}
+	else
+	{
+		IAnjutaIterable* begin = ianjuta_editor_get_line_begin_position (editor, line -1 , NULL);
+		IAnjutaIterable* end = ianjuta_editor_get_line_end_position (editor, line -1 , NULL);
+
+		if (spaces_only (editor, begin, end))
+		{
+			set_line_indentation (editor, line -1, 0, 0);
+		}
+		g_object_unref (begin);
+		g_object_unref (end);
+	}
 	
 	iter = ianjuta_editor_get_line_begin_position (editor, line, NULL);
-	
+
 	if (is_iter_inside_string (iter))
 	{
 		line_indent = get_line_indentation (editor, line - 1);
@@ -1242,20 +1304,22 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 	else
 	{
 		line_indent = get_line_indentation_base (plugin, editor, line,
-												 &incomplete_statement, &colon_indent);
+												 &incomplete_statement, 
+												 line_indent_spaces,
+												 &colon_indent);
 	}
-	
+
 	if (colon_indent)
 	{
 		/* If the last non-whitespace character in the line is ":" then
 		 * we remove the extra colon_indent
 		 */
-		gchar ch;
 		end_iter = ianjuta_editor_get_line_end_position (editor, line, NULL);
+		gchar ch;
 		while (ianjuta_iterable_previous (end_iter, NULL))
 		{
 			ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (end_iter),
-											   0, NULL);
+			                                   0, NULL);
 			if (ch == ':')
 			{
 				line_indent -= INDENT_SIZE;
@@ -1324,6 +1388,7 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 		else if (ch == '#')
 		{
 			line_indent = 0;
+			*line_indent_spaces = 0;
 		}
 		else if (!isspace (ch))
 		{
@@ -1335,6 +1400,7 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 	}
 	while (ianjuta_iterable_next (iter, NULL));
 	g_object_unref (iter);
+	
 	return line_indent;
 }
 
@@ -1351,7 +1417,7 @@ on_editor_char_inserted_cpp (IAnjutaEditor *editor,
 	iter = ianjuta_iterable_clone (insert_pos, NULL);
 	
 	/* If autoindent is enabled*/
-	if (anjuta_preferences_get_int (plugin->prefs, PREF_INDENT_AUTOMATIC))
+	if (plugin->smart_indentation)
 	{
 	
 		/* DEBUG_PRINT ("Char added at position %d: '%c'", insert_pos, ch); */	
@@ -1404,17 +1470,19 @@ on_editor_char_inserted_cpp (IAnjutaEditor *editor,
 		{
 			gint insert_line;
 			gint line_indent;
+			gint line_indent_spaces;
 		
 			ianjuta_document_begin_undo_action (IANJUTA_DOCUMENT(editor), NULL);
 			initialize_indentation_params (plugin);
+			
 			insert_line = ianjuta_editor_get_lineno (editor, NULL);
-			line_indent = get_line_auto_indentation (plugin, editor, insert_line);
-			set_line_indentation (editor, insert_line, line_indent);
+			line_indent = get_line_auto_indentation (plugin, editor, insert_line, &line_indent_spaces);
+			set_line_indentation (editor, insert_line, line_indent, line_indent_spaces);
 			ianjuta_document_end_undo_action (IANJUTA_DOCUMENT(editor), NULL);
 		}
 	}
 	
-	if (anjuta_preferences_get_int (plugin->prefs, PREF_BRACE_AUTOCOMPLETION))
+	if (anjuta_preferences_get_bool (plugin->prefs, PREF_BRACE_AUTOCOMPLETION))
 	{
 		if (ch == '[' || ch == '(')
 		{
@@ -1428,8 +1496,8 @@ on_editor_char_inserted_cpp (IAnjutaEditor *editor,
 			/* If the previous char is a ' we don't have to autocomplete */
 			if (*prev_char != '\'')
 			{
+				
 				ianjuta_iterable_next (iter, NULL);
-		
 				switch (ch)
 				{
 					case '[': ianjuta_document_begin_undo_action (IANJUTA_DOCUMENT (editor), NULL);
@@ -1456,14 +1524,14 @@ on_editor_char_inserted_cpp (IAnjutaEditor *editor,
 		{
 			gchar *prev_char;
 			IAnjutaIterable *previous;
-			
+
 			previous = ianjuta_iterable_clone (iter, NULL);
 			ianjuta_iterable_previous (previous, NULL);
 			prev_char = ianjuta_editor_get_text (editor, previous, iter, NULL);
 
 			/* First iter*/
 			ianjuta_iterable_next (iter, NULL);
-		
+
 			/*
 			 * If the character is " we have to decide if we need insert
 			 * another " or we have to skip the character
@@ -1553,9 +1621,6 @@ install_support (CppJavaPlugin *lang_plugin)
 	}
 	
 	initialize_indentation_params (lang_plugin);
-	/* Disable editor intern auto-indent */
-	ianjuta_editor_set_auto_indent (IANJUTA_EDITOR(lang_plugin->current_editor),
-								    FALSE, NULL);
 	
 	if (IANJUTA_IS_EDITOR_ASSIST (lang_plugin->current_editor) &&
 		!g_str_equal (lang_plugin->current_language, "Vala"))
@@ -1825,10 +1890,12 @@ on_auto_indent (GtkAction *action, gpointer data)
 	
 	for (insert_line = line_start; insert_line <= line_end; insert_line++)
 	{
+		gint line_indent_spaces = 0;
 		line_indent = get_line_auto_indentation (lang_plugin, editor,
-												 insert_line);
+												 insert_line,
+												 &line_indent_spaces);
 		/* DEBUG_PRINT ("Line indent for line %d = %d", insert_line, line_indent); */
-		set_line_indentation (editor, insert_line, line_indent);
+		set_line_indentation (editor, insert_line, line_indent, line_indent_spaces);
 	}
 	ianjuta_document_end_undo_action (IANJUTA_DOCUMENT(editor), NULL);
 }
@@ -1839,7 +1906,7 @@ on_auto_complete (GtkAction *action, gpointer data)
 	CppJavaPlugin *lang_plugin;
 	lang_plugin = ANJUTA_PLUGIN_CPP_JAVA (data);
 	if (lang_plugin->assist)
-		cpp_java_assist_check (lang_plugin->assist, TRUE, TRUE);
+		cpp_java_assist_check (lang_plugin->assist, TRUE, TRUE, FALSE);
 }
 
 static GtkActionEntry actions[] = {
@@ -1991,14 +2058,19 @@ static void
 ipreferences_merge (IAnjutaPreferences* ipref, AnjutaPreferences* prefs,
 					GError** e)
 {
-	GladeXML* gxml;
+	GError* error = NULL;
+	GtkBuilder* bxml = gtk_builder_new ();
 		
 	/* Add preferences */
-	gxml = glade_xml_new (PREFS_GLADE, "preferences_dialog", NULL);
-	anjuta_preferences_add_page (prefs,
-								 gxml, "preferences", _("C/C++/Java/Vala"),
+	if (!gtk_builder_add_from_file (bxml, PREFS_BUILDER, &error))
+	{
+		g_warning ("Couldn't load builder file: %s", error->message);
+		g_error_free (error);
+	}
+	anjuta_preferences_add_from_builder (prefs,
+								 bxml, "preferences", _("C/C++/Java/Vala"),
 								 ICON_FILE);
-	g_object_unref (gxml);
+	g_object_unref (bxml);
 }
 
 static void
