@@ -27,8 +27,9 @@ char *s = N_("Berkeley Software Distribution License (BSD)");
 char *s = N_("Comma separated, other plugins that this plugin depends on. It could be either primary interface name or plugin location (library:class)");
 char *s = N_("Configure external packages");
 char *s = N_("Configure external packages:");
-char *s = N_("Create a template glade interface file");
-char *s = N_("Create glade interface file");
+char *s = N_("Create Gtk Builder interface file");
+char *s = N_("Create a template Gtk Builder interface file");
+char *s = N_("Create a template gtk builder interface file");
 char *s = N_("Description:");
 char *s = N_("Destination:");
 char *s = N_("Display description of the applet");
